package view;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.Random;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import model.BesetztException;
import model.IModel;
import model.Spieler;
import model.Model; // nur zu Testzwecken


/**
 * Einfaches graphisches View fr Gobang mit MVC
 * Stellt das Spielbrett und eine Informationstafel auf einem swing.JFrame dar.
 * Auf der Informationstafel wird whrend des Spiels "Stein setzen: " (mit "O" oder "X" je nach dem nchsten Spielers) dargestellt.
 * Am Ende des Spiels steht dort "o gewinnt" bzw. "x gewinnt", je nach Sieger. Die Koordinaten der Endpunkte der Siegerlinie werden auch angegeben. 
 * Die Brettgre (Anzahl der Felder) und die Feldgre (in Pixeln) wird als Konstruktorparameter angegeben.
 * Beenden erfolgt durch Schlieen des Fensters.
 * @author Prof. Dr. Andreas Solymosi, Beuth-Hochschule fr Technik, (c) 2012, solymosi@tfh-berlin.de
 * @version 5/1/12 
 */
public class View extends JPanel implements IView {
	private static final long serialVersionUID = 1L;
	private static final String steinSetzen = "Stein setzen: ",
		feldBesetzt = "Feld besetzt -" + steinSetzen,
		gewinnt = " gewinnt";
	
	private int hhe, breite; // dargestellte Brettgre
	private IModel model; // Konstruktorparameter
    private Panel panel; // fr die Darstellung des Bretts
    private JLabel label; // Informationstafel

    /*
     * @param model
     * @param hhe Anzahl der Felder
     * @param breite Anzahl der Felder
     * @param feldgre in Pixeln; wird nur lokal gebraucht
     */
    public View(final IModel model, int hhe, int breite, int feldgre) {
        this.model = model;
        this.hhe = hhe;
        this.breite = breite;

        super.setLayout(new BorderLayout());
        this.label = new JLabel(steinSetzen + model.nchster());
        super.add(label, BorderLayout.NORTH);        
        this.panel = new Panel();
        panel.setPreferredSize(new Dimension(feldgre*hhe, feldgre*breite));
        super.add(panel, BorderLayout.CENTER);

        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.getContentPane().add(this);
        frame.pack();
        frame.setVisible(true);
    }
	@Override
	public Dimension brettgre() {
		return new Dimension(hhe, breite);
	}

	@Override
    public void neuZeichnen() {
		if (model.sieger() != Spieler.LEER)
			label.setText(model.sieger() + gewinnt);
		else
			label.setText(steinSetzen + model.nchster());			
        this.repaint();
    }
    public void feldBesetzt(int x, int y) {
		label.setText(feldBesetzt + model.nchster() + " " + x + "/" + y);			
        this.repaint();    	
    }

    private class Panel extends JPanel { // Brett
		private static final long serialVersionUID = 1L;
		@Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            int dx = this.getWidth() / hhe;
            int dy = this.getHeight() / breite;            
            // Gitter zeichnen:
            g.setColor(Color.BLACK);
            for (int zeile = 1; zeile < hhe; zeile++)
                for (int spalte = 1; spalte < breite; spalte++) { // nur zwischen den Feldern
                    g.drawLine(zeile*dx, 0, zeile*dx, getHeight());
                    g.drawLine(0, spalte*dy, getWidth(), spalte*dy);
                }
            // auf besetzte Felder Figuren zeichnen: 
            for (int zeile = 0; zeile < hhe; zeile++)
                for (int spalte = 0; spalte < breite; spalte++) {                	
                	// Figuren an der Siegerlinie rot zeichnen, andere schwarz:
                	g.setColor(model.siegerlinie(zeile, spalte) ? Color.red : Color.black);
                    if (model.besetztVon(zeile, spalte) == Spieler.X) { // Kreuz zeichnen
                    	final int x0 = zeile*dx, y0 = spalte*dy;  
                        g.drawLine(x0, y0, x0+dx, y0+dy);
                        g.drawLine(x0, y0+dy, x0+dx, y0);                    	
                    }
                    else if (model.besetztVon(zeile, spalte) == Spieler.O) // Kreis zeichnen
                    	g.drawOval(zeile*dx, spalte*dy, dx, dy);
                }
        }
    }    

	// Testreiber: mit zufallsgesteuertem Model wenn Kommandozeilenparameter besetzt, ansonsten mit Dummy-Model 
	public static void main(String[] kzp) throws InterruptedException {
    	 if (kzp.length == 0) { // autonomer Test mit Dummy-Model
    			final int breite = 5, hhe = 6, feldgre = 100;
    			IView view = new View(new IModel(){
    				public Spieler besetztVon(int zeile, int spalte) {
    					return Spieler.X;
    				}
    				public Spieler nchster() {
    					return Spieler.O;
    				}
    				public void start() {
    				}
    				public void steinSetzen(int zeile, int spalte) throws BesetztException {
    				}
    				public Spieler sieger() {
    					return Spieler.LEER;
    				}
					public boolean siegerlinie(int zeile, int spalte) {
						return false;
					}}, hhe, breite, feldgre);
    			view.neuZeichnen();
    	}    		 
    	else {
    		// Zufallsgesteuerter Test mit Model
			int hhe = kzp.length > 0 ? Integer.parseInt(kzp[0]) : 20, // Bretthhe, Anzahl der Felder, Standardwert = 20
				breite = kzp.length > 1 ? Integer.parseInt(kzp[1]) : 20, // Brettbreite, Anzahl der Felder, Standardwert = 20
				feldgre = kzp.length > 2 ? Integer.parseInt(kzp[1]) : 50; // Feldgre, Anzahl der Pixel, Standardwert = 20
			boolean xFngtAn = kzp.length > 3 ? kzp[3].charAt(0) == 'X' : false; // Standardwert: O fngt an
			int pause = kzp.length > 4 ? Integer.parseInt(kzp[4]) : 100;  // Standardwert: 0,1 Sekunden
	
			IModel model = new Model(xFngtAn);
			View view = new View(model, hhe, breite, feldgre); // Testling
	
			Random r = new Random(new java.util.Date().getTime());
			while (true) {
				int zeile = Math.abs(r.nextInt() % hhe), spalte = Math.abs(r.nextInt() % breite); // Zufallszahl zwischen 0 und 20 (bzw. hhe/breite)		
				try {
					model.steinSetzen(zeile, spalte);
				} catch (BesetztException e) {
					// 
				}
				view.neuZeichnen();
				Thread.sleep(pause);
				if (model.sieger() != Spieler.LEER) { // Spiel ist zu Ende
					view.label.setText(model.sieger() + gewinnt + ": " + zeile + "#" + spalte); // privater Zugriff
					break;
				}
			}
    	}
	}
}